#region License Information
/* HeuristicLab
 * Copyright (C) 2002-2013 Heuristic and Evolutionary Algorithms Laboratory (HEAL)
 *
 * This file is part of HeuristicLab.
 *
 * HeuristicLab is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * HeuristicLab is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with HeuristicLab. If not, see <http://www.gnu.org/licenses/>.
 */
#endregion

using HeuristicLab.Common;
using HeuristicLab.Core;
using HeuristicLab.Parameters;
using HeuristicLab.Persistence.Default.CompositeSerializers.Storable;

namespace HeuristicLab.Operators {
  /// <summary>
  /// An operator which removes a variable from the current scope.
  /// </summary>
  [Item("Remover", "An operator which removes a variable from the current scope.")]
  [StorableClass]
  public sealed class Remover : SingleSuccessorOperator {

    private ScopeParameter CurrentScopeParameter {
      get { return (ScopeParameter)Parameters["CurrentScope"]; }
    }

    public ValueLookupParameter<IItem> TargetParameter {
      get { return (ValueLookupParameter<IItem>)Parameters["Target"]; }
    }

    public IScope CurrentScope {
      get { return CurrentScopeParameter.ActualValue; }
    }

    [StorableConstructor]
    private Remover(bool deserializing) : base(deserializing) { }
    private Remover(Remover original, Cloner cloner)
      : base(original, cloner) {
    }
    public Remover()
      : base() {
       Parameters.Add(new ValueLookupParameter<IItem>("Target", "The target variable that will be removed."));
       Parameters.Add(new ScopeParameter("CurrentScope", "The current scope from which one or all sub-scopes should be removed."));
    }

    public override IDeepCloneable Clone(Cloner cloner) {
      return new Remover(this, cloner);
    }

    public override IOperation Apply() {
      CurrentScope.Variables.Remove(TargetParameter.ActualName);
      return base.Apply();
    }
  }
}
